import os
from tinydb import TinyDB, Query
from database import DatabaseConnector
from typing import Self
from datetime import datetime

class User:
    db_connector =  DatabaseConnector().get_table("users")

    # Constructor
    def __init__(self, name, id) -> None:
        self.name = name
        self.id = id
        
    # String representation of the class
    def __str__(self):
        return f'User (Object) {self.id}, ({self.name})'

    # String representation of the class
    def __repr__(self):
        return self.__str__()
    
    def store_data(self):
        print("Storing data...")
        self.last_update == datetime.now()

        query = Query()
        # upsert: https://tinydb.readthedocs.io/en/latest/usage.html#upserting-data
        result = self.db_connector.upsert(self.__dict__, query.id == self.id)
        if result:
            print("Data updated.")
        else:
            print("Data inserted.")
    
    def delete(self):
        print("Deleting data...")
        query = Query()
        if self.db_connector.remove(query.id == self.id):
            print("Data deleted.")
        else:
            print("Data not found.")

    # Class method that can be called without an instance of the class to construct an instance of the class
    @classmethod
    def find_by_attribute(cls, by_attribute: str, attribute_value: str, num_to_return=1) -> Self | list[Self]:
        # Load data from the database and create an instance of the Device class
        query = Query()
        result = cls.db_connector.search(query[by_attribute] == attribute_value)

        if result:
            data = result[:num_to_return]
            results = [cls(d['name'], d['id']) for d in data]
            return results if num_to_return > 1 else results[0]
        else:
            return None

    @classmethod
    def find_all(cls) -> list[Self]:
        # Load all data from the database and create instances of the Device class
        results = []
        for data in cls.db_connector.all():
            results = [cls(d['name'], d['id']) for d in data]
        return results

if __name__ == "__main__":
    # Create a device
    user1 = User("User One", "one@mci.edu")
    user2 = User("User Two", "two@mci.edu") 
    user3 = User("User Three", "three@mci.edu") 
    user1.store_data()
    user2.store_data()
    user3.store_data()
    user4 = User("User Four", "four@mci.edu") 
    user4.store_data()

    loaded_user = User.find_by_attribute("id", "three@mci.edu")
    if loaded_user:
        print(f"Loaded: {loaded_user}")
    else:
        print("User not found.")